@@ -1,12 +1,12 @@
-/**
- *  Copyright (C) 2019 Ryszard Wiśniewski <brut.alll@gmail.com>
- *  Copyright (C) 2019 Connor Tumbleson <connor.tumbleson@gmail.com>
+/*
+ *  Copyright (C) 2010 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2010 Connor Tumbleson <connor.tumbleson@gmail.com>
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -17,7 +17,7 @@
 package brut.androlib.res;
 
 import brut.androlib.AndrolibException;
-import brut.androlib.ApkOptions;
+import brut.androlib.options.BuildOptions;
 import brut.androlib.err.CantFindFrameworkResException;
 import brut.androlib.meta.MetaInfo;
 import brut.androlib.meta.PackageInfo;
@@ -26,12 +26,12 @@ import brut.androlib.res.data.*;
 import brut.androlib.res.decoder.*;
 import brut.androlib.res.decoder.ARSCDecoder.ARSCData;
 import brut.androlib.res.decoder.ARSCDecoder.FlagsOffset;
-import brut.directory.*;
 import brut.androlib.res.util.ExtMXSerializer;
 import brut.androlib.res.util.ExtXmlSerializer;
 import brut.androlib.res.xml.ResValuesXmlSerializable;
 import brut.androlib.res.xml.ResXmlPatcher;
 import brut.common.BrutException;
+import brut.directory.*;
 import brut.util.*;
 import org.apache.commons.io.IOUtils;
 import org.xmlpull.v1.XmlSerializer;
@@ -44,9 +44,6 @@ import java.util.zip.ZipEntry;
 import java.util.zip.ZipFile;
 import java.util.zip.ZipOutputStream;
 
-/**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- */
 final public class AndrolibResources {
     public ResTable getResTable(ExtFile apkFile) throws AndrolibException {
         return getResTable(apkFile, true);
@@ -65,23 +62,16 @@ final public class AndrolibResources {
             throws AndrolibException {
         LOGGER.info("Loading resource table...");
         ResPackage[] pkgs = getResPackagesFromApk(apkFile, resTable, sKeepBroken);
-        ResPackage pkg = null;
+        ResPackage pkg;
 
         switch (pkgs.length) {
             case 1:
                 pkg = pkgs[0];
                 break;
             case 2:
-                if (pkgs[0].getName().equals("android")) {
-                    LOGGER.warning("Skipping \"android\" package group");
-                    pkg = pkgs[1];
-                    break;
-                } else if (pkgs[0].getName().equals("com.htc")) {
-                    LOGGER.warning("Skipping \"htc\" package group");
-                    pkg = pkgs[1];
-                    break;
-                }
-
+                LOGGER.warning("Skipping package group: " + pkgs[0].getName());
+                pkg = pkgs[1];
+                break;
             default:
                 pkg = selectPkgWithMostResSpecs(pkgs);
                 break;
@@ -95,8 +85,7 @@ final public class AndrolibResources {
         return pkg;
     }
 
-    public ResPackage selectPkgWithMostResSpecs(ResPackage[] pkgs)
-            throws AndrolibException {
+    public ResPackage selectPkgWithMostResSpecs(ResPackage[] pkgs) {
         int id = 0;
         int value = 0;
         int index = 0;
@@ -132,7 +121,7 @@ final public class AndrolibResources {
         }
 
         if (pkg.getId() != id) {
-            throw new AndrolibException("Expected pkg of id: " + String.valueOf(id) + ", got: " + pkg.getId());
+            throw new AndrolibException("Expected pkg of id: " + id + ", got: " + pkg.getId());
         }
 
         resTable.addPackage(pkg, false);
@@ -142,7 +131,7 @@ final public class AndrolibResources {
     public void decodeManifest(ResTable resTable, ExtFile apkFile, File outDir)
             throws AndrolibException {
 
-        Duo<ResFileDecoder, AXmlResourceParser> duo = getManifestFileDecoder();
+        Duo<ResFileDecoder, AXmlResourceParser> duo = getManifestFileDecoder(false);
         ResFileDecoder fileDecoder = duo.m1;
 
         // Set ResAttrDecoder
@@ -189,7 +178,7 @@ final public class AndrolibResources {
     public void decodeManifestWithResources(ResTable resTable, ExtFile apkFile, File outDir)
             throws AndrolibException {
 
-        Duo<ResFileDecoder, AXmlResourceParser> duo = getResFileDecoder();
+        Duo<ResFileDecoder, AXmlResourceParser> duo = getManifestFileDecoder(true);
         ResFileDecoder fileDecoder = duo.m1;
         ResAttrDecoder attrDecoder = duo.m2.getAttrDecoder();
 
@@ -317,8 +306,8 @@ final public class AndrolibResources {
         return Integer.toString(target);
     }
 
-    private File createDoNotCompressExtensionsFile(ApkOptions apkOptions) throws AndrolibException {
-        if (apkOptions.doNotCompress == null || apkOptions.doNotCompress.isEmpty()) {
+    private File createDoNotCompressExtensionsFile(BuildOptions buildOptions) throws AndrolibException {
+        if (buildOptions.doNotCompress == null || buildOptions.doNotCompress.isEmpty()) {
             return null;
         }
 
@@ -328,7 +317,7 @@ final public class AndrolibResources {
             doNotCompressFile.deleteOnExit();
 
             BufferedWriter fileWriter = new BufferedWriter(new FileWriter(doNotCompressFile));
-            for (String extension : apkOptions.doNotCompress) {
+            for (String extension : buildOptions.doNotCompress) {
                 fileWriter.write(extension);
                 fileWriter.newLine();
             }
@@ -369,11 +358,11 @@ final public class AndrolibResources {
             cmd.add("-o");
             cmd.add(resourcesZip.getAbsolutePath());
 
-            if (apkOptions.verbose) {
+            if (buildOptions.verbose) {
                 cmd.add("-v");
             }
 
-            if (apkOptions.noCrunch) {
+            if (buildOptions.noCrunch) {
                 cmd.add("--no-crunch");
             }
 
@@ -444,24 +433,24 @@ final public class AndrolibResources {
             cmd.add("--enable-sparse-encoding");
         }
 
-        if (apkOptions.isFramework) {
+        if (buildOptions.isFramework) {
             cmd.add("-x");
         }
 
-        if (apkOptions.doNotCompress != null && !customAapt) {
+        if (buildOptions.doNotCompress != null && !customAapt) {
             // Use custom -e option to avoid limits on commandline length.
             // Can only be used when custom aapt binary is not used.
-            String extensionsFilePath = createDoNotCompressExtensionsFile(apkOptions).getAbsolutePath();
+            String extensionsFilePath = createDoNotCompressExtensionsFile(buildOptions).getAbsolutePath();
             cmd.add("-e");
             cmd.add(extensionsFilePath);
-        } else if (apkOptions.doNotCompress != null) {
-            for (String file : apkOptions.doNotCompress) {
+        } else if (buildOptions.doNotCompress != null) {
+            for (String file : buildOptions.doNotCompress) {
                 cmd.add("-0");
                 cmd.add(file);
             }
         }
 
-        if (!apkOptions.resourcesAreCompressed) {
+        if (!buildOptions.resourcesAreCompressed) {
             cmd.add("-0");
             cmd.add("arsc");
         }
@@ -486,7 +475,7 @@ final public class AndrolibResources {
             cmd.add(rawDir.getAbsolutePath());
         }
 
-        if (apkOptions.verbose) {
+        if (buildOptions.verbose) {
             cmd.add("-v");
         }
 
@@ -509,16 +498,16 @@ final public class AndrolibResources {
 
         cmd.add("p");
 
-        if (apkOptions.verbose) { // output aapt verbose
+        if (buildOptions.verbose) { // output aapt verbose
             cmd.add("-v");
         }
-        if (apkOptions.updateFiles) {
+        if (buildOptions.updateFiles) {
             cmd.add("-u");
         }
-        if (apkOptions.debugMode) { // inject debuggable="true" into manifest
+        if (buildOptions.debugMode) { // inject debuggable="true" into manifest
             cmd.add("--debug-mode");
         }
-        if (apkOptions.noCrunch) {
+        if (buildOptions.noCrunch) {
             cmd.add("--no-crunch");
         }
         // force package id so that some frameworks build with correct id
@@ -566,24 +555,24 @@ final public class AndrolibResources {
         cmd.add("-F");
         cmd.add(apkFile.getAbsolutePath());
 
-        if (apkOptions.isFramework) {
+        if (buildOptions.isFramework) {
             cmd.add("-x");
         }
 
-        if (apkOptions.doNotCompress != null && !customAapt) {
+        if (buildOptions.doNotCompress != null && !customAapt) {
             // Use custom -e option to avoid limits on commandline length.
             // Can only be used when custom aapt binary is not used.
-            String extensionsFilePath = createDoNotCompressExtensionsFile(apkOptions).getAbsolutePath();
+            String extensionsFilePath = createDoNotCompressExtensionsFile(buildOptions).getAbsolutePath();
             cmd.add("-e");
             cmd.add(extensionsFilePath);
-        } else if (apkOptions.doNotCompress != null) {
-            for (String file : apkOptions.doNotCompress) {
+        } else if (buildOptions.doNotCompress != null) {
+            for (String file : buildOptions.doNotCompress) {
                 cmd.add("-0");
                 cmd.add(file);
             }
         }
 
-        if (!apkOptions.resourcesAreCompressed) {
+        if (!buildOptions.resourcesAreCompressed) {
             cmd.add("-0");
             cmd.add("arsc");
         }
@@ -621,9 +610,9 @@ final public class AndrolibResources {
     public void aaptPackage(File apkFile, File manifest, File resDir, File rawDir, File assetDir, File[] include)
             throws AndrolibException {
 
-        String aaptPath = apkOptions.aaptPath;
+        String aaptPath = buildOptions.aaptPath;
         boolean customAapt = !aaptPath.isEmpty();
-        List<String> cmd = new ArrayList<String>();
+        List<String> cmd = new ArrayList<>();
 
         try {
             String aaptCommand = AaptManager.getAaptExecutionCommand(aaptPath, getAaptBinaryFile());
@@ -633,7 +622,7 @@ final public class AndrolibResources {
             cmd.add(AaptManager.getAaptBinaryName(getAaptVersion()));
         }
 
-        if (apkOptions.isAapt2()) {
+        if (buildOptions.isAapt2()) {
             aapt2Package(apkFile, manifest, resDir, rawDir, assetDir, include, cmd, customAapt);
             return;
         }
@@ -644,7 +633,7 @@ final public class AndrolibResources {
             throws AndrolibException {
 
         try {
-            ZipUtils.zipFolders(rawDir, apkFile, assetDir, apkOptions.doNotCompress);
+            ZipUtils.zipFolders(rawDir, apkFile, assetDir, buildOptions.doNotCompress);
         } catch (IOException | BrutException ex) {
             throw new AndrolibException(ex);
         }
@@ -671,6 +660,13 @@ final public class AndrolibResources {
                 return ResConfigFlags.SDK_P;
             case "Q":
                 return ResConfigFlags.SDK_Q;
+            case "R":
+                return ResConfigFlags.SDK_R;
+            case "S":
+                return ResConfigFlags.SDK_S;
+            case "T":
+            case "Tiramisu":
+                return ResConfigFlags.SDK_DEVELOPMENT;
             default:
                 return Integer.parseInt(sdkVersion);
         }
@@ -705,17 +701,19 @@ final public class AndrolibResources {
         axmlParser.setAttrDecoder(new ResAttrDecoder());
         decoders.setDecoder("xml", new XmlPullStreamDecoder(axmlParser, getResXmlSerializer()));
 
-        return new Duo<ResFileDecoder, AXmlResourceParser>(new ResFileDecoder(decoders), axmlParser);
+        return new Duo<>(new ResFileDecoder(decoders), axmlParser);
     }
 
-    public Duo<ResFileDecoder, AXmlResourceParser> getManifestFileDecoder() {
+    public Duo<ResFileDecoder, AXmlResourceParser> getManifestFileDecoder(boolean withResources) {
         ResStreamDecoderContainer decoders = new ResStreamDecoderContainer();
 
-        AXmlResourceParser axmlParser = new AXmlResourceParser();
-
+        AXmlResourceParser axmlParser = new AndroidManifestResourceParser();
+        if (withResources) {
+            axmlParser.setAttrDecoder(new ResAttrDecoder());
+        }
         decoders.setDecoder("xml", new XmlPullStreamDecoder(axmlParser,getResXmlSerializer()));
 
-        return new Duo<ResFileDecoder, AXmlResourceParser>(new ResFileDecoder(decoders), axmlParser);
+        return new Duo<>(new ResFileDecoder(decoders), axmlParser);
     }
 
     public ExtMXSerializer getResXmlSerializer() {
@@ -781,15 +779,10 @@ final public class AndrolibResources {
             throws AndrolibException {
         try {
             Directory dir = apkFile.getDirectory();
-            BufferedInputStream bfi = new BufferedInputStream(dir.getFileInput("resources.arsc"));
-            try {
+            try (BufferedInputStream bfi = new BufferedInputStream(dir.getFileInput("resources.arsc"))) {
                 return ARSCDecoder.decode(bfi, false, keepBroken, resTable).getPackages();
-            } finally {
-                try {
-                    bfi.close();
-                } catch (IOException ignored) {}
             }
-        } catch (DirectoryException ex) {
+        } catch (DirectoryException | IOException ex) {
             throw new AndrolibException("Could not load resources.arsc from file: " + apkFile, ex);
         }
     }
@@ -806,13 +799,13 @@ final public class AndrolibResources {
             }
         }
 
-        apk = new File(dir, String.valueOf(id) + ".apk");
+        apk = new File(dir, id + ".apk");
         if (apk.exists()) {
             return apk;
         }
 
         if (id == 1) {
-            try (InputStream in = AndrolibResources.class.getResourceAsStream("/brut/androlib/android-framework.jar");
+            try (InputStream in = getAndroidFrameworkResourcesAsStream();
                  OutputStream out = new FileOutputStream(apk)) {
                 IOUtils.copy(in, out);
                 return apk;
@@ -834,7 +827,7 @@ final public class AndrolibResources {
             LOGGER.warning("Can't empty framework directory, no file found at: " + apk.getAbsolutePath());
         } else {
             try {
-                if (apk.exists() && dir.listFiles().length > 1 && ! apkOptions.forceDeleteFramework) {
+                if (apk.exists() && dir.listFiles().length > 1 && ! buildOptions.forceDeleteFramework) {
                     LOGGER.warning("More than default framework detected. Please run command with `--force` parameter to wipe framework directory.");
                 } else {
                     for (File file : dir.listFiles()) {
@@ -850,8 +843,22 @@ final public class AndrolibResources {
         }
     }
 
+    public void listFrameworkDirectory() throws AndrolibException {
+        File dir = getFrameworkDir();
+        if (dir == null) {
+            LOGGER.severe("No framework directory found. Nothing to list.");
+            return;
+        }
+
+        for (File file : Objects.requireNonNull(dir.listFiles())) {
+            if (file.isFile() && file.getName().endsWith(".apk")) {
+                LOGGER.info(file.getName());
+            }
+        }
+    }
+
     public void installFramework(File frameFile) throws AndrolibException {
-        installFramework(frameFile, apkOptions.frameworkTag);
+        installFramework(frameFile, buildOptions.frameworkTag);
     }
 
     public void installFramework(File frameFile, String tag)
@@ -872,8 +879,8 @@ final public class AndrolibResources {
             ARSCData arsc = ARSCDecoder.decode(new ByteArrayInputStream(data), true, true);
             publicizeResources(data, arsc.getFlagsOffsets());
 
-            File outFile = new File(getFrameworkDir(), String.valueOf(arsc
-                    .getOnePackage().getId())
+            File outFile = new File(getFrameworkDir(), arsc
+                .getOnePackage().getId()
                     + (tag == null ? "" : '-' + tag)
                     + ".apk");
 
@@ -883,11 +890,12 @@ final public class AndrolibResources {
             crc.update(data);
             entry = new ZipEntry("resources.arsc");
             entry.setSize(data.length);
+            entry.setMethod(ZipOutputStream.STORED);
             entry.setCrc(crc.getValue());
             out.putNextEntry(entry);
             out.write(data);
             out.closeEntry();
-            
+
             //Write fake AndroidManifest.xml file to support original aapt
             entry = zip.getEntry("AndroidManifest.xml");
             if (entry != null) {
@@ -930,8 +938,7 @@ final public class AndrolibResources {
         publicizeResources(arsc, ARSCDecoder.decode(new ByteArrayInputStream(arsc), true, true).getFlagsOffsets());
     }
 
-    public void publicizeResources(byte[] arsc, FlagsOffset[] flagsOffsets)
-            throws AndrolibException {
+    public void publicizeResources(byte[] arsc, FlagsOffset[] flagsOffsets) {
         for (FlagsOffset flags : flagsOffsets) {
             int offset = flags.offset + 3;
             int end = offset + 4 * flags.count;
@@ -950,8 +957,8 @@ final public class AndrolibResources {
         String path;
 
         // if a framework path was specified on the command line, use it
-        if (apkOptions.frameworkFolderLocation != null) {
-            path = apkOptions.frameworkFolderLocation;
+        if (buildOptions.frameworkFolderLocation != null) {
+            path = buildOptions.frameworkFolderLocation;
         } else {
             File parentPath = new File(System.getProperty("user.home"));
 
@@ -976,14 +983,16 @@ final public class AndrolibResources {
 
         if (! dir.exists()) {
             if (! dir.mkdirs()) {
-                if (apkOptions.frameworkFolderLocation != null) {
+                if (buildOptions.frameworkFolderLocation != null) {
                     LOGGER.severe("Can't create Framework directory: " + dir);
                 }
-                throw new AndrolibException("Can't create directory: " + dir);
+                throw new AndrolibException(String.format(
+                        "Can't create directory: (%s). Pass a writable path with --frame-path {DIR}. ", dir
+                ));
             }
         }
 
-        if (apkOptions.frameworkFolderLocation == null) {
+        if (buildOptions.frameworkFolderLocation == null) {
             if (! dir.canWrite()) {
                 LOGGER.severe(String.format("WARNING: Could not write to (%1$s), using %2$s instead...",
                         dir.getAbsolutePath(), System.getProperty("java.io.tmpdir")));
@@ -1001,24 +1010,20 @@ final public class AndrolibResources {
     private File getAaptBinaryFile() throws AndrolibException {
         try {
             if (getAaptVersion() == 2) {
-                return AaptManager.getAppt2();
+                return AaptManager.getAapt2();
             }
-            return AaptManager.getAppt1();
+            return AaptManager.getAapt1();
         } catch (BrutException ex) {
             throw new AndrolibException(ex);
         }
     }
 
     private int getAaptVersion() {
-        return apkOptions.isAapt2() ? 2 : 1;
+        return buildOptions.isAapt2() ? 2 : 1;
     }
 
-    public File getAndroidResourcesFile() throws AndrolibException {
-        try {
-            return Jar.getResourceAsFile("/brut/androlib/android-framework.jar");
-        } catch (BrutException ex) {
-            throw new AndrolibException(ex);
-        }
+    public InputStream getAndroidFrameworkResourcesAsStream() {
+        return Jar.class.getResourceAsStream("/brut/androlib/android-framework.jar");
     }
 
     public void close() throws IOException {
@@ -1027,7 +1032,7 @@ final public class AndrolibResources {
         }
     }
 
-    public ApkOptions apkOptions;
+    public BuildOptions buildOptions;
 
     // TODO: dirty static hack. I have to refactor decoding mechanisms.
     public static boolean sKeepBroken = false;
