@@ -1,12 +1,12 @@
-/**
- *  Copyright (C) 2019 Ryszard Wiśniewski <brut.alll@gmail.com>
- *  Copyright (C) 2019 Connor Tumbleson <connor.tumbleson@gmail.com>
+/*
+ *  Copyright (C) 2010 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2010 Connor Tumbleson <connor.tumbleson@gmail.com>
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -17,23 +17,25 @@
 package brut.androlib.res.decoder;
 
 import android.util.TypedValue;
-import brut.androlib.Androlib;
 import brut.androlib.AndrolibException;
 import brut.androlib.res.data.*;
 import brut.androlib.res.data.value.*;
 import brut.util.Duo;
-import brut.androlib.res.data.ResTable;
 import brut.util.ExtDataInput;
 import com.google.common.io.LittleEndianDataInputStream;
-import java.io.*;
-import java.math.BigInteger;
-import java.util.*;
-import java.util.logging.Logger;
 import org.apache.commons.io.input.CountingInputStream;
 
-/**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- */
+import java.io.DataInput;
+import java.io.EOFException;
+import java.io.IOException;
+import java.io.InputStream;
+import java.math.BigInteger;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.List;
+import java.util.logging.Logger;
+
 public class ARSCDecoder {
     public static ARSCData decode(InputStream arscStream, boolean findFlagsOffsets, boolean keepBroken)
             throws AndrolibException {
@@ -57,7 +59,7 @@ public class ARSCDecoder {
     private ARSCDecoder(InputStream arscStream, ResTable resTable, boolean storeFlagsOffsets, boolean keepBroken) {
         arscStream = mCountIn = new CountingInputStream(arscStream);
         if (storeFlagsOffsets) {
-            mFlagsOffsets = new ArrayList<FlagsOffset>();
+            mFlagsOffsets = new ArrayList<>();
         } else {
             mFlagsOffsets = null;
         }
@@ -123,12 +125,19 @@ public class ARSCDecoder {
         mPkg = new ResPackage(mResTable, id, name);
 
         nextChunk();
-        while (mHeader.type == Header.TYPE_LIBRARY) {
-            readLibraryType();
-        }
-
-        while (mHeader.type == Header.TYPE_SPEC_TYPE) {
-            readTableTypeSpec();
+        boolean flag = true;
+        while (flag) {
+            switch (mHeader.type) {
+                case Header.TYPE_LIBRARY:
+                    readLibraryType();
+                    break;
+                case Header.TYPE_SPEC_TYPE:
+                    readTableTypeSpec();
+                    break;
+                default:
+                    flag = false;
+                    break;
+            }
         }
 
         return mPkg;
@@ -242,7 +251,7 @@ public class ARSCDecoder {
         }
 
         mType = flags.isInvalid && !mKeepBroken ? null : mPkg.getOrCreateConfig(flags);
-        HashMap<Integer, EntryData> offsetsToEntryData = new HashMap<Integer, EntryData>();
+        HashMap<Integer, EntryData> offsetsToEntryData = new HashMap<>();
 
         for (int offset : entryOffsets) {
             if (offset == -1 || offsetsToEntryData.containsKey(offset)) {
@@ -300,12 +309,12 @@ public class ARSCDecoder {
             if (spec.isDummyResSpec()) {
                 removeResSpec(spec);
 
-                spec = new ResResSpec(resId, mSpecNames.getString(specNamesId), mPkg, mTypeSpec);
+                spec = new ResResSpec(resId, mSpecNames.getString(specNamesId), entryData.mFlags, mPkg, mTypeSpec);
                 mPkg.addResSpec(spec);
                 mTypeSpec.addResSpec(spec);
             }
         } else {
-            spec = new ResResSpec(resId, mSpecNames.getString(specNamesId), mPkg, mTypeSpec);
+            spec = new ResResSpec(resId, mSpecNames.getString(specNamesId), entryData.mFlags, mPkg, mTypeSpec);
             mPkg.addResSpec(spec);
             mTypeSpec.addResSpec(spec);
         }
@@ -318,12 +327,11 @@ public class ARSCDecoder {
             if (mKeepBroken) {
                 mType.addResource(res, true);
                 spec.addResource(res, true);
-                LOGGER.warning(String.format("Duplicate Resource Detected. Ignoring duplicate: %s", res.toString()));
+                LOGGER.warning(String.format("Duplicate Resource Detected. Ignoring duplicate: %s", res));
             } else {
                 throw ex;
             }
         }
-        mPkg.addResource(res);
     }
 
     private ResBagValue readComplexEntry() throws IOException, AndrolibException {
@@ -339,12 +347,10 @@ public class ARSCDecoder {
             resId = mIn.readInt();
             resValue = readValue();
 
-            if (resValue instanceof ResScalarValue) {
-                items[i] = new Duo<Integer, ResScalarValue>(resId, (ResScalarValue) resValue);
-            } else {
+            if (!(resValue instanceof ResScalarValue)) {
                 resValue = new ResStringValue(resValue.toString(), resValue.getRawIntValue());
-                items[i] = new Duo<Integer, ResScalarValue>(resId, (ResScalarValue) resValue);
             }
+            items[i] = new Duo<>(resId, (ResScalarValue) resValue);
         }
 
         return factory.bagFactory(parent, items, mTypeSpec);
@@ -464,7 +470,7 @@ public class ARSCDecoder {
                 colorMode, isInvalid, size);
     }
 
-    private char[] unpackLanguageOrRegion(byte in0, byte in1, char base) throws AndrolibException {
+    private char[] unpackLanguageOrRegion(byte in0, byte in1, char base) {
         // check high bit, if so we have a packed 3 letter code
         if (((in0 >> 7) & 1) == 1) {
             int first = in1 & 0x1F;
@@ -478,7 +484,7 @@ public class ARSCDecoder {
         return new char[] { (char) in0, (char) in1 };
     }
 
-    private String readScriptOrVariantChar(int length) throws AndrolibException, IOException {
+    private String readScriptOrVariantChar(int length) throws IOException {
         StringBuilder string = new StringBuilder(16);
 
         while(length-- != 0) {
@@ -505,7 +511,7 @@ public class ARSCDecoder {
                 continue;
             }
 
-            ResResSpec spec = new ResResSpec(new ResID(resId | i), "APKTOOL_DUMMY_" + Integer.toHexString(i), mPkg, mTypeSpec);
+            ResResSpec spec = new ResResSpec(new ResID(resId | i), "APKTOOL_DUMMY_" + Integer.toHexString(i), ENTRY_FLAG_PUBLIC, mPkg, mTypeSpec);
 
             // If we already have this resID dont add it again.
             if (! mPkg.hasResSpec(new ResID(resId | i))) {
@@ -516,10 +522,11 @@ public class ARSCDecoder {
                     mType = mPkg.getOrCreateConfig(new ResConfigFlags());
                 }
 
-                ResValue value = new ResBoolValue(false, 0, null);
-                ResResource res = new ResResource(mType, spec, value);
+                // We are going to make dummy attributes a null reference (@null) now instead of a boolean false.
+                // This is because aapt2 is much more strict when it comes to what we can put in an application.
+                ResValue value = new ResReferenceValue(mPkg, 0, "");
 
-                mPkg.addResource(res);
+                ResResource res = new ResResource(mType, spec, value);
                 mType.addResource(res);
                 spec.addResource(res);
             }
@@ -565,10 +572,10 @@ public class ARSCDecoder {
     private int mResId;
     private int mTypeIdOffset = 0;
     private boolean[] mMissingResSpecs;
-    private HashMap<Integer, ResTypeSpec> mResTypeSpecs = new HashMap<>();
+    private final HashMap<Integer, ResTypeSpec> mResTypeSpecs = new HashMap<>();
 
     private final static short ENTRY_FLAG_COMPLEX = 0x0001;
-    private final static short ENTRY_FLAG_PUBLIC = 0x0002;
+    public final static short ENTRY_FLAG_PUBLIC = 0x0002;
     private final static short ENTRY_FLAG_WEAK = 0x0004;
 
     public static class Header {
