@@ -1,12 +1,12 @@
-/**
- *  Copyright (C) 2019 Ryszard Wiśniewski <brut.alll@gmail.com>
- *  Copyright (C) 2019 Connor Tumbleson <connor.tumbleson@gmail.com>
+/*
+ *  Copyright (C) 2010 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2010 Connor Tumbleson <connor.tumbleson@gmail.com>
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
  *  You may obtain a copy of the License at
  *
- *       http://www.apache.org/licenses/LICENSE-2.0
+ *       https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -23,28 +23,24 @@ import brut.androlib.res.data.ResID;
 import brut.androlib.res.xml.ResXmlEncoders;
 import brut.util.ExtDataInput;
 import com.google.common.io.LittleEndianDataInputStream;
+import org.xmlpull.v1.XmlPullParserException;
 import java.io.DataInput;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
 import java.util.logging.Level;
 import java.util.logging.Logger;
-import org.xmlpull.v1.XmlPullParserException;
 
 /**
- * @author Ryszard Wiśniewski <brut.alll@gmail.com>
- * @author Dmitry Skiba
+ * Binary xml files parser.
  *
- *         Binary xml files parser.
- *
- *         Parser has only two states: (1) Operational state, which parser
- *         obtains after first successful call to next() and retains until
- *         open(), close(), or failed call to next(). (2) Closed state, which
- *         parser obtains after open(), close(), or failed call to next(). In
- *         this state methods return invalid values or throw exceptions.
- *
- *         TODO: * check all methods in closed state
+ * Parser has only two states: (1) Operational state, which parser
+ * obtains after first successful call to next() and retains until
+ * open(), close(), or failed call to next(). (2) Closed state, which
+ * parser obtains after open(), close(), or failed call to next(). In
+ * this state methods return invalid values or throw exceptions.
  *
+ * TODO: * check all methods in closed state
  */
 public class AXmlResourceParser implements XmlResourceParser {
 
@@ -214,18 +210,18 @@ public class AXmlResourceParser implements XmlResourceParser {
     }
 
     @Override
-    public int getNamespaceCount(int depth) throws XmlPullParserException {
+    public int getNamespaceCount(int depth) {
         return m_namespaces.getAccumulatedCount(depth);
     }
 
     @Override
-    public String getNamespacePrefix(int pos) throws XmlPullParserException {
+    public String getNamespacePrefix(int pos) {
         int prefix = m_namespaces.getPrefix(pos);
         return m_strings.getString(prefix);
     }
 
     @Override
-    public String getNamespaceUri(int pos) throws XmlPullParserException {
+    public String getNamespaceUri(int pos) {
         int uri = m_namespaces.getUri(pos);
         return m_strings.getString(uri);
     }
@@ -296,7 +292,7 @@ public class AXmlResourceParser implements XmlResourceParser {
         if (value.length() == 0) {
             ResID resourceId = new ResID(getAttributeNameResource(index));
             if (resourceId.package_ == PRIVATE_PKG_ID) {
-                value = getNonDefaultNamespaceUri();
+                value = getNonDefaultNamespaceUri(offset);
             } else {
                 value = android_ns;
             }
@@ -305,15 +301,19 @@ public class AXmlResourceParser implements XmlResourceParser {
         return value;
     }
 
-    private String getNonDefaultNamespaceUri() {
-        int offset = m_namespaces.getCurrentCount() + 1;
-        String prefix = m_strings.getString(m_namespaces.get(offset, true));
-
-        if (! prefix.equalsIgnoreCase("android")) {
-            return  m_strings.getString(m_namespaces.get(offset, false));
+    private String getNonDefaultNamespaceUri(int offset) {
+        String prefix = m_strings.getString(m_namespaces.getPrefix(offset));
+        if (prefix != null) {
+            return  m_strings.getString(m_namespaces.getUri(offset));
         }
 
-        return android_ns;
+        // If we are here. There is some clever obfuscation going on. Our reference points to the namespace are gone.
+        // Normally we could take the index * attributeCount to get an offset.
+        // That would point to the URI in the StringBlock table, but that is empty.
+        // We have the namespaces that can't be touched in the opening tag.
+        // Though no known way to correlate them at this time.
+        // So return the res-auto namespace.
+        return "http://schemas.android.com/apk/res-auto";
     }
 
     @Override
@@ -339,15 +339,15 @@ public class AXmlResourceParser implements XmlResourceParser {
 
         // some attributes will return "", we must rely on the resource_id and refer to the frameworks
         // to match the resource id to the name. ex: 0x101021C = versionName
-        if (value.length() != 0 && !android_ns.equals(getAttributeNamespace(index))) {
-            return value;
-        } else {
+        if (value.length() == 0 || android_ns.equals(getAttributeNamespace(index))) {
             try {
-                value = mAttrDecoder.decodeManifestAttr(getAttributeNameResource(index));
-            } catch (AndrolibException e) {
-            }
-            return value;
+                int resourceId = getAttributeNameResource(index);
+                if (resourceId != 0) {
+                    value = mAttrDecoder.decodeManifestAttr(getAttributeNameResource(index));
+                }
+            } catch (AndrolibException | NullPointerException ignored) {}
         }
+        return value;
     }
 
     @Override
@@ -517,8 +517,7 @@ public class AXmlResourceParser implements XmlResourceParser {
 
     // ///////////////////////////////// dummies
     @Override
-    public void setInput(InputStream stream, String inputEncoding)
-            throws XmlPullParserException {
+    public void setInput(InputStream stream, String inputEncoding) {
         open(stream);
     }
 
@@ -538,12 +537,12 @@ public class AXmlResourceParser implements XmlResourceParser {
     }
 
     @Override
-    public boolean isEmptyElementTag() throws XmlPullParserException {
+    public boolean isEmptyElementTag() {
         return false;
     }
 
     @Override
-    public boolean isWhitespace() throws XmlPullParserException {
+    public boolean isWhitespace() {
         return false;
     }
 
@@ -602,17 +601,12 @@ public class AXmlResourceParser implements XmlResourceParser {
             m_data = new int[32];
         }
 
-        public final void reset() {
+        public void reset() {
             m_dataLength = 0;
-            m_count = 0;
             m_depth = 0;
         }
 
-        public final int getTotalCount() {
-            return m_count;
-        }
-
-        public final int getCurrentCount() {
+        public int getCurrentCount() {
             if (m_dataLength == 0) {
                 return 0;
             }
@@ -620,7 +614,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             return m_data[offset];
         }
 
-        public final int getAccumulatedCount(int depth) {
+        public int getAccumulatedCount(int depth) {
             if (m_dataLength == 0 || depth < 0) {
                 return 0;
             }
@@ -637,7 +631,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             return accumulatedCount;
         }
 
-        public final void push(int prefix, int uri) {
+        public void push(int prefix, int uri) {
             if (m_depth == 0) {
                 increaseDepth();
             }
@@ -649,38 +643,9 @@ public class AXmlResourceParser implements XmlResourceParser {
             m_data[offset + 1] = uri;
             m_data[offset + 2] = count + 1;
             m_dataLength += 2;
-            m_count += 1;
         }
 
-        public final boolean pop(int prefix, int uri) {
-            if (m_dataLength == 0) {
-                return false;
-            }
-            int offset = m_dataLength - 1;
-            int count = m_data[offset];
-            for (int i = 0, o = offset - 2; i != count; ++i, o -= 2) {
-                if (m_data[o] != prefix || m_data[o + 1] != uri) {
-                    continue;
-                }
-                count -= 1;
-                if (i == 0) {
-                    m_data[o] = count;
-                    o -= (1 + count * 2);
-                    m_data[o] = count;
-                } else {
-                    m_data[offset] = count;
-                    offset -= (1 + 2 + count * 2);
-                    m_data[offset] = count;
-                    System.arraycopy(m_data, o + 2, m_data, o, m_dataLength - o);
-                }
-                m_dataLength -= 2;
-                m_count -= 1;
-                return true;
-            }
-            return false;
-        }
-
-        public final boolean pop() {
+        public boolean pop() {
             if (m_dataLength == 0) {
                 return false;
             }
@@ -695,31 +660,26 @@ public class AXmlResourceParser implements XmlResourceParser {
             offset -= (1 + count * 2);
             m_data[offset] = count;
             m_dataLength -= 2;
-            m_count -= 1;
             return true;
         }
 
-        public final int getPrefix(int index) {
+        public int getPrefix(int index) {
             return get(index, true);
         }
 
-        public final int getUri(int index) {
+        public int getUri(int index) {
             return get(index, false);
         }
 
-        public final int findPrefix(int uri) {
+        public int findPrefix(int uri) {
             return find(uri, false);
         }
 
-        public final int findUri(int prefix) {
-            return find(prefix, true);
-        }
-
-        public final int getDepth() {
+        public int getDepth() {
             return m_depth;
         }
 
-        public final void increaseDepth() {
+        public void increaseDepth() {
             ensureDataCapacity(2);
             int offset = m_dataLength;
             m_data[offset] = 0;
@@ -728,7 +688,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             m_depth += 1;
         }
 
-        public final void decreaseDepth() {
+        public void decreaseDepth() {
             if (m_dataLength == 0) {
                 return;
             }
@@ -738,7 +698,6 @@ public class AXmlResourceParser implements XmlResourceParser {
                 return;
             }
             m_dataLength -= 2 + count * 2;
-            m_count -= count;
             m_depth -= 1;
         }
 
@@ -753,7 +712,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             m_data = newData;
         }
 
-        private final int find(int prefixOrUri, boolean prefix) {
+        private int find(int prefixOrUri, boolean prefix) {
             if (m_dataLength == 0) {
                 return -1;
             }
@@ -777,7 +736,7 @@ public class AXmlResourceParser implements XmlResourceParser {
             return -1;
         }
 
-        private final int get(int index, boolean prefix) {
+        private int get(int index, boolean prefix) {
             if (m_dataLength == 0 || index < 0) {
                 return -1;
             }
@@ -800,15 +759,10 @@ public class AXmlResourceParser implements XmlResourceParser {
 
         private int[] m_data;
         private int m_dataLength;
-        private int m_count;
         private int m_depth;
     }
 
-    final StringBlock getStrings() {
-        return m_strings;
-    }
-
-    private final int getAttributeOffset(int index) {
+    private int getAttributeOffset(int index) {
         if (m_event != START_TAG) {
             throw new IndexOutOfBoundsException("Current event is not START_TAG.");
         }
@@ -819,7 +773,7 @@ public class AXmlResourceParser implements XmlResourceParser {
         return offset;
     }
 
-    private final int findAttribute(String namespace, String attribute) {
+    private int findAttribute(String namespace, String attribute) {
         if (m_strings == null || attribute == null) {
             return -1;
         }
@@ -837,7 +791,7 @@ public class AXmlResourceParser implements XmlResourceParser {
         return -1;
     }
 
-    private final void resetEventInfo() {
+    private void resetEventInfo() {
         m_event = -1;
         m_lineNumber = -1;
         m_name = -1;
@@ -848,7 +802,7 @@ public class AXmlResourceParser implements XmlResourceParser {
         m_styleAttribute = -1;
     }
 
-    private final void doNext() throws IOException {
+    private void doNext() throws IOException {
         // Delayed initialization.
         if (m_strings == null) {
             m_reader.skipCheckInt(CHUNK_AXML_FILE, CHUNK_AXML_FILE_BROKEN);
@@ -984,8 +938,8 @@ public class AXmlResourceParser implements XmlResourceParser {
     private boolean m_operational = false;
     private StringBlock m_strings;
     private int[] m_resourceIDs;
-    private NamespaceStack m_namespaces = new NamespaceStack();
-    private String android_ns = "http://schemas.android.com/apk/res/android";
+    private final NamespaceStack m_namespaces = new NamespaceStack();
+    private final String android_ns = "http://schemas.android.com/apk/res/android";
     private boolean m_decreaseDepth;
     private int m_event;
     private int m_lineNumber;
